/************************** (C) COPYRIGHT 2021  *****************
* File Name       : 
* Author            : 
* Version          : Ver 0.1
* Date               : 2021/10/26
* Description    : 
*
* Revision History
* Date          Version       Name          					Description
* 2021/10/06  	0.1          Oanny        					Proto.
*******************************************************************************/
#include "nrf_drv_twi.h"
#include "nrf_delay.h"
#include "nrf_gpio.h"

#include "usr_system.h"
#include "usr_i2c.h"
#include "usr_uart.h"
#include "usr_timer.h"

#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>
#include <math.h>

/* TWI instance. */
static const nrf_drv_twi_t m_twi = NRF_DRV_TWI_INSTANCE(TWI_INSTANCE_ID);
/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
void I2C_Init(void)
{
    ret_code_t err_code;

		/* I2C Interrupt */
		nrf_gpio_cfg_sense_input(I2C_INT_PIN, NRF_GPIO_PIN_PULLUP, NRF_GPIO_PIN_SENSE_LOW);

    const nrf_drv_twi_config_t twi_config = {
       .scl                = I2C_SCL_PIN,
       .sda                = I2C_SDA_PIN,
       .frequency          = NRF_DRV_TWI_FREQ_100K,//NRF_DRV_TWI_FREQ_400K
       .interrupt_priority = APP_IRQ_PRIORITY_HIGH,
       .clear_bus_init     = false
    };
    err_code = nrf_drv_twi_init(&m_twi, &twi_config, NULL, NULL);
    APP_ERROR_CHECK(err_code);

    nrf_drv_twi_enable(&m_twi);
}

tSensor stSensor;
#if defined(I2C_MAX30100)
/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
 //Setup MAX30102
  byte ledBrightness = 0x1F; //Options: 0=Off to 255=50mA
  byte sampleAverage = 8; //Options: 1, 2, 4, 8, 16, 32
  byte ledMode = 2; //Options: 1 = Red only, 2 = Red + IR, 3 = Red + IR + Green
  //Options: 1 = IR only, 2 = Red + IR on MH-ET LIVE MAX30102 board
  int sampleRate = 400; //Options: 50, 100, 200, 400, 800, 1000, 1600, 3200
  int pulseWidth = 411; //Options: 69, 118, 215, 411
  int adcRange = 4096; //Options: 2048, 4096, 8192, 16384
  // Set up the wanted parameters
  particleSensor.setup(ledBrightness, sampleAverage, ledMode, sampleRate, pulseWidth, adcRange); //Configure sensor with these settings
}
*******************************************************************************/
void MAX30100_Init(void)
{
    uint8_t cfg[1] = {0, };
    uint8_t cfg_reset[1] = {0, };
		uint32_t SensorTimeout = SENSOR_TIMEOUT;

		MAX30100_I2C_register_read(HEARTRATE_MODE_CONFIG, cfg_reset, 1);
    
    cfg_reset[0] |= 0x40;    
		MAX30100_I2C_register_write(HEARTRATE_SPO2_CONFIG, cfg_reset, 1);
    // wait for the RESET bit to clear itself
    while(1){
				MAX30100_I2C_register_read(HEARTRATE_MODE_CONFIG, cfg_reset, 1);
				if((SensorTimeout--) == 0) break;
        if((cfg_reset[0] & 0x40) == 0) break;
    }
    // read old and set new mode
    MAX30100_I2C_register_read(HEARTRATE_MODE_CONFIG, cfg, 1);		
    cfg[0] = (cfg[0] & ~0x07) | HEARTRATE_HR_ONLY | HEARTRATE_SPO2_EN | HEARTRATE_TEMP_EN;
    MAX30100_I2C_register_write(HEARTRATE_MODE_CONFIG, cfg, 1);
		
    MAX30100_I2C_register_read(HEARTRATE_SPO2_CONFIG, cfg, 1);   // read SpO2 configuration
    cfg[0] |= (1 << 6);                                         // enable High Resolution SpO2
    cfg[0] |= SET_HEARTRATE_SAMPLES;//HEARTRATE_SAMPLES_100;                               // set samples per second
    cfg[0] |= SET_HEARTRATE_PULSE_WIDTH;                         // set 16-bit ADC resolution
    MAX30100_I2C_register_write(HEARTRATE_SPO2_CONFIG, cfg, 1);  // send new SpO2 configuration

    cfg[0] = 0x00;
    cfg[0] |= HEARTRATE_IR_CURRENT_500;                           // set IR LED current to be 50 mA
    cfg[0] |= HEARTRATE_RED_CURRENT_240;//HEARTRATE_RED_CURRENT_271;                          // set RED LED current to be 24 mA
    MAX30100_I2C_register_write(HEARTRATE_LED_CONFIG, cfg, 1);   // send new LED current control config

    // interrupt settings
    MAX30100_I2C_register_read(HEARTRATE_INT_ENABLE, cfg, 1);		
    cfg[0] |= HEARTRATE_ENA_HR_RDY;
    cfg[0] |= HEARTRATE_ENA_SO2_RDY;
    cfg[0] |= HEARTRATE_ENA_TEP_RDY;		
    // send new interrupt settings
    MAX30100_I2C_register_write(HEARTRATE_INT_ENABLE, cfg, 1);					
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
ret_code_t MAX30100_I2C_register_write(uint8_t reg_addr, uint8_t * p_tx_data, uint8_t bytes)
{
    ret_code_t ret;
    uint8_t tx_data[bytes+1];
#if 1
		tx_data[0] = reg_addr;
    for(uint8_t i = 0 ; i<bytes ; i++){
        tx_data[i+1] = p_tx_data[i];
    }
#else		
    tx_data[0] = reg_addr;    
    memcpy(&tx_data[1], p_tx_data, bytes);
#endif		
    ret = nrf_drv_twi_tx(&m_twi, MAX30100_ADDRESS, tx_data, sizeof(tx_data), true);
		if (NRF_SUCCESS != ret){
        return false;
    }
		return true;
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
ret_code_t MAX30100_I2C_register_read(uint8_t reg_addr,  uint8_t * p_rx_data, uint32_t bytes)
{   
    ret_code_t ret;
	
    ret = nrf_drv_twi_tx(&m_twi, MAX30100_ADDRESS, &reg_addr, 1, true);
		if (NRF_SUCCESS != ret){
        return false;
    }
    ret = nrf_drv_twi_rx(&m_twi, MAX30100_ADDRESS, p_rx_data, bytes);
		if (NRF_SUCCESS != ret){
        return false;
    }
		return true;
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
uint8_t MAX30100_get_status(void)
{
    uint8_t read_data;
	
    MAX30100_I2C_register_read(HEARTRATE_INT_STATUS, &read_data, 1);
    return read_data;
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
uint8_t MAX30100_data_ready(void)
{
	uint8_t status;

	status = (GPIO_ReadInputDataBit(I2C_INT_PIN) == 0) && ((MAX30100_get_status() & 0x20) != 0);
//	DEBUGPRINT("status %d, %d\r\n", (GPIO_ReadInputDataBit(I2C_INT_PIN)), status);
	return status;
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
uint8_t MAX30100_read_ir_red(uint16_t *ir_buff, uint16_t *red_buff)
{
  uint8_t i, wr_ptr = 0, rd_ptr = 0, sample_num = 1, samples[4];

	MAX30100_I2C_register_read(HEARTRATE_FIFO_WRITE_PTR, &wr_ptr, 1);
	MAX30100_I2C_register_read(HEARTRATE_FIFO_READ_PTR, &rd_ptr, 1);

	if((abs(16 + wr_ptr - rd_ptr) % 16) > 0){
    sample_num = abs(16 + wr_ptr - rd_ptr) % 16;
//		DEBUGPRINT("====> %d, %d, %d\r\n", wr_ptr, rd_ptr, sample_num);
	}
	if(sample_num > 0){
		for ( i = 0; i < sample_num; ++i ){
			MAX30100_I2C_register_read(HEARTRATE_FIFO_DATA_REG, samples, 4);
			((uint8_t *)ir_buff  )[0]  = samples[1];
			((uint8_t *)ir_buff++)[1]  = samples[0];

			((uint8_t *)red_buff  )[0]  = samples[3];
			((uint8_t *)red_buff++)[1]  = samples[2];			
		}
	}
	else{
		MAX30100_I2C_register_read(HEARTRATE_FIFO_DATA_REG, samples, 4);
		*ir_buff = 0;
		*red_buff = 0;
	}
	return sample_num;
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
uint8_t MAX30100_get_revision_id(void)
{
	uint8_t read_data;

	MAX30100_I2C_register_read(HEARTRATE_REVISION_ID, &read_data, 1);
	return read_data;
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
uint8_t MAX30100_get_part_id(void)
{
    uint8_t read_data;
	
    MAX30100_I2C_register_read(HEARTRATE_PART_ID, &read_data, 1);
    return read_data;
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
uint8_t MAX30100_get_config (void)
{
    uint8_t read_data;
	
    MAX30100_I2C_register_read(HEARTRATE_MODE_CONFIG, &read_data, 1);
    return read_data;
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
void MAX30100_set_config (uint8_t cfg)
{
    MAX30100_I2C_register_write(HEARTRATE_MODE_CONFIG, &cfg, 1);
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
void MAX30100_set_interrupt (uint8_t intrpts)
{
    MAX30100_I2C_register_write(HEARTRATE_MODE_CONFIG, &intrpts, 1);
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
float MAX30100_read_temp (float *temp_value)
{	
	uint8_t Buff[2];

  MAX30100_I2C_register_read(HEARTRATE_TEMP_FRACTION, Buff, 2); 
	MAX30100_temp_enable(true);
	
	*temp_value = Buff[0];
	*temp_value += (((float)(Buff[1] & 0x0F)) / 16);
	
	return *temp_value;	
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
void MAX30100_shut_down (bool enable)
{
  uint8_t temp;
	
  MAX30100_I2C_register_read(HEARTRATE_MODE_CONFIG, &temp, 1);
  if (enable == true){
    temp |= (1 << 7);
  }
  else{
    temp &= ~(1 << 7);
  }

  MAX30100_I2C_register_write(HEARTRATE_MODE_CONFIG, &temp, 1);
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
void MAX30100_reset (void)
{
  uint8_t temp;
	
  MAX30100_I2C_register_read(HEARTRATE_MODE_CONFIG, &temp, 1);

  temp |= (1 << 6);
  MAX30100_I2C_register_write(HEARTRATE_MODE_CONFIG, &temp, 1);
}

#define MODE_TEMP_EN	(1<<3)
/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
void MAX30100_temp_enable (bool enable)
{
  uint8_t temp;
  MAX30100_I2C_register_read(HEARTRATE_MODE_CONFIG, &temp, 1);

  if(enable == true){
    temp |= MODE_TEMP_EN;
  }
  else{
    temp &= ~MODE_TEMP_EN;
  }
  MAX30100_I2C_register_write(HEARTRATE_MODE_CONFIG, &temp, 1);
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
int8_t MAX30100_set_mode (uint8_t mode)
{
  uint8_t temp;

  if (mode > 7){
    return -1;
  }
  MAX30100_I2C_register_read(HEARTRATE_MODE_CONFIG, &temp, 1);

  temp |= mode;
  MAX30100_I2C_register_write(HEARTRATE_MODE_CONFIG, &temp, 1);
  return 0;
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
void MAX30100_spo2_high_res_enable (bool enable)
{
  uint8_t temp;
	
  MAX30100_I2C_register_read(HEARTRATE_SPO2_CONFIG, &temp, 1);
  if (enable == true){
    temp |= (1 << 6);
  }
  else{
    temp &= ~(1 << 6);
  }
  MAX30100_I2C_register_write(HEARTRATE_SPO2_CONFIG, &temp, 1);
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
uint8_t MAX30100_spo2_set_sample_rate (uint8_t sample_rate)
{
  uint8_t temp;
	
  if (sample_rate > 7){
    return 1;
  }
  MAX30100_I2C_register_read(HEARTRATE_SPO2_CONFIG, &temp, 1);
  temp |= (sample_rate << 2);

  MAX30100_I2C_register_write(HEARTRATE_SPO2_CONFIG, &temp, 1);
  return 0;
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
uint8_t MAX30100_set_led_pulse_width (uint8_t value)
{
  uint8_t temp;
	
  if (value > 3){
    return 1;
  }
  MAX30100_I2C_register_read(HEARTRATE_SPO2_CONFIG, &temp, 1);

  temp |= value;
  MAX30100_I2C_register_write(HEARTRATE_SPO2_CONFIG, &temp, 1);
  return 0;
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
uint8_t MAX30100_set_red_current_level (uint8_t level)
{
  uint8_t temp;
	
  if (level > 15){
    return 1;
  }
  MAX30100_I2C_register_read(HEARTRATE_LED_CONFIG, &temp, 1);

  temp |= (level << 4);
  MAX30100_I2C_register_write(HEARTRATE_LED_CONFIG, &temp, 1);
  return 0;
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
uint8_t MAX30100_set_ir_current_level (uint8_t level)
{
  uint8_t temp;
	
  if (level > 15){
    return 1;
  }
  MAX30100_I2C_register_read(HEARTRATE_LED_CONFIG, &temp, 1);

  temp |= level;
  MAX30100_I2C_register_write(HEARTRATE_LED_CONFIG, &temp, 1);
  return 0;
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
void MAX30100_fifo_almost_full_int_enable (bool enable)
{
  uint8_t temp;
	
  MAX30100_I2C_register_read(HEARTRATE_INT_ENABLE, &temp, 1);
  if (enable == true){
    temp |= (1 << 7);
  }
  else{
    temp &= ~(1 << 7);
  }   
  MAX30100_I2C_register_write(HEARTRATE_INT_ENABLE, &temp, 1);
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
void MAX30100_temp_ready_int_enable (bool enable)
{
  uint8_t temp;
	
  MAX30100_I2C_register_read(HEARTRATE_INT_ENABLE, &temp, 1);
  if (enable == true){
    temp |= (1 << 6);
  }
  else{
    temp &= ~(1 << 6);
  }
  MAX30100_I2C_register_write(HEARTRATE_INT_ENABLE, &temp, 1);
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
void MAX30100_heartrate_data_ready_int_enable (bool enable)
{
  uint8_t temp;
  MAX30100_I2C_register_read(HEARTRATE_INT_ENABLE, &temp, 1);
  if (enable == true){
    temp |= (1 << 5);
  }
  else{
    temp &= ~(1 << 5);
  }
  MAX30100_I2C_register_write(HEARTRATE_INT_ENABLE, &temp, 1);
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
void MAX30100_heartrate_spo2_data_ready_int_enable (bool enable)
{
  uint8_t temp;
	
  MAX30100_I2C_register_read(HEARTRATE_INT_ENABLE, &temp, 1);
  if(enable == true){
    temp |= (1 << 4);
  }
  else{
    temp &= ~(1 << 4);
  }
  MAX30100_I2C_register_write(HEARTRATE_INT_ENABLE, &temp, 1);
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
void MAX30100_printRegisters(void)
{
	uint8_t rx_data;
	
	MAX30100_I2C_register_read(HEARTRATE_INT_STATUS, &rx_data, 1);
  DEBUGPRINT("MAX30100_INT_STATUS 0x%2X\r\n", rx_data);
	MAX30100_I2C_register_read(HEARTRATE_INT_ENABLE, &rx_data, 1);
  DEBUGPRINT("MAX30100_INT_ENABLE 0x%2X\r\n", rx_data);
	MAX30100_I2C_register_read(HEARTRATE_FIFO_WRITE_PTR, &rx_data, 1);
  DEBUGPRINT("MAX30100_FIFO_WR_PTR 0x%2X\r\n", rx_data);
	MAX30100_I2C_register_read(HEARTRATE_OVER_FLOW_CNT, &rx_data, 1);
  DEBUGPRINT("MAX30100_OVRFLOW_CTR 0x%2X\r\n", rx_data);
	MAX30100_I2C_register_read(HEARTRATE_FIFO_READ_PTR, &rx_data, 1);
  DEBUGPRINT("MAX30100_FIFO_RD_PTR 0x%2X\r\n", rx_data);
	MAX30100_I2C_register_read(HEARTRATE_FIFO_DATA_REG, &rx_data, 1);
  DEBUGPRINT("MAX30100_FIFO_DATA 0x%2X\r\n", rx_data);
	MAX30100_I2C_register_read(HEARTRATE_MODE_CONFIG, &rx_data, 1);
  DEBUGPRINT("MAX30100_MODE_CONFIG 0x%2X\r\n", rx_data);
	MAX30100_I2C_register_read(HEARTRATE_SPO2_CONFIG, &rx_data, 1);
  DEBUGPRINT("MAX30100_SPO2_CONFIG 0x%2X\r\n", rx_data);
	MAX30100_I2C_register_read(HEARTRATE_LED_CONFIG, &rx_data, 1);
  DEBUGPRINT("MAX30100_LED_CONFIG 0x%2X\r\n", rx_data);
	MAX30100_I2C_register_read(HEARTRATE_TEMP_INTEGER, &rx_data, 1);
  DEBUGPRINT("MAX30100_TEMP_INTG 0x%2X\r\n", rx_data);
	MAX30100_I2C_register_read(HEARTRATE_TEMP_FRACTION, &rx_data, 1);
  DEBUGPRINT("MAX30100_TEMP_FRAC 0x%2X\r\n", rx_data);
	MAX30100_I2C_register_read(HEARTRATE_REVISION_ID, &rx_data, 1);
  DEBUGPRINT("MAX30100_REV_ID 0x%2X\r\n", rx_data);
	MAX30100_I2C_register_read(HEARTRATE_PART_ID, &rx_data, 1);
  DEBUGPRINT("MAX30100_PART_ID 0x%2X\r\n", rx_data);
}

																						 
#define FINGER_ON 30000
#define MAX_BPS 180
#define MIN_BPS 45

static double fbpmrate = 0.95; // low pass filter coefficient for HRM in bpm
static uint32_t crosstime = 0; //falling edge , zero crossing time in msec
static uint32_t crosstime_prev = 0;//previous falling edge , zero crossing time in msec
static double bpm = 40.0;
static double ebpm = 40.0;
static double eir = 0.0; //estimated lowpass filtered IR signal to find falling edge without notch
static double firrate = 0.85; //IR filter coefficient to remove notch ,should be smaller than frate
static double eir_prev = 0.0;
/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
double HRM_estimator(double fir, double aveir, uint8_t* fingeron)
{
  int CTdiff;

  //Heart Rate Monitor by finding falling edge
  eir = eir * firrate + fir * (1.0 - firrate); //estimated IR : low pass filtered IR signal

  if ( ((eir - aveir) * (eir_prev - aveir) < 0 ) && ((eir - aveir) < 0.0)) { //find zero cross at falling edge
    crosstime = Tick_Cnt;//millis(); //system time in msec of falling edge
    CTdiff = crosstime-crosstime_prev;
#if 0
    DEBUGPRINT("CRT: ");
    DEBUGPRINT(crosstime);
    DEBUGPRINT(" CRT_PREV: ");
    DEBUGPRINT(crosstime_prev);
    DEBUGPRINT(" CTdiff: ");
    DEBUGPRINT("%d\r\n", CTdiff);
#endif
    //if ( ((crosstime - crosstime_prev ) > (60 * 1000 / MAX_BPS)) &&((crosstime - crosstime_prev ) < (60 * 1000 / MIN_BPS)) ) {
    if ( ( CTdiff > 333 ) && ( CTdiff < 1333 ) ) {
      bpm = 60.0 * 1000.0 / (double)(crosstime - crosstime_prev) ; //get bpm
      // Serial.println("crossed");
      ebpm = ebpm * fbpmrate + (1.0 - fbpmrate) * bpm; //estimated bpm by low pass filtered
      
      if (aveir > FINGER_ON) {
				*fingeron = 1;
//        DEBUGPRINT("FINGER_ON %f\r\n", (200-(100.0-ESpO2)*10.0));//when SpO2=80% BLIPSOUND drops 200Hz to indicate anormaly
      }
			else{
				*fingeron = 0;
			}
    }
		else {
      //Serial.println("faild to find falling edge");
    }
    crosstime_prev = crosstime;
  }
   eir_prev = eir;
   return (ebpm);
}

#define MINIMUM_SPO2 80.0
#define FINGER_ON_IR 50000 // if ir signal is lower than this , it indicates your finger is not on the sensor

uint8_t chFingeron = 0;
uint16_t ir, red;
int Ebpm = 60;
int i = 0;
int Num = 50; //calculate SpO2 by this sampling interval
double fred, fir;
double SpO2 = 0; //raw SpO2 before low pass filtered
double aveir = 0;
double avered = 0; 
double sumirrms = 0;
double sumredrms = 0;

double ESpO2 = 95.0;//initial value of estimated SpO2
double FSpO2 = 0.7; //filter factor for estimated SpO2
double frate = 0.95; // .95 default low pass filter for IR/red LED value to eliminate AC component
/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
void HRM_Display(void)
{
	if(MAX30100_data_ready()){
		MAX30100_read_ir_red(&ir, &red); 			
		i++;
		fred = (double)red; // 
		fir = (double)ir;
		avered = avered * frate + fred * (1.0 - frate);//average red level by low pass filter
		aveir = aveir * frate + fir * (1.0 - frate); //average IR level by low pass filter
		sumredrms += (fred - avered) * (fred - avered); //square sum of alternate component of red level
		sumirrms += (fir - aveir) * (fir - aveir);//square sum of alternate component of IR level
													
		// True Result of MAX30102
		Ebpm = (int) HRM_estimator(fir, aveir, &chFingeron); //Ebpm is estimated BPM
		if (ir < FINGER_ON_IR)
			ESpO2 = MINIMUM_SPO2; //indicator for finger detached

//			if((ir > 150) && (red > 150) && ((i % Num) == 0)){						
		if((chFingeron == 1) && ((i % Num) == 0)){				
			double R = (sqrt(sumredrms) / avered) / (sqrt(sumirrms) / aveir); // ratio of (redrms/avered) / (irrms/aveir)
			SpO2 = -23.3 * (R - 0.4) + 100; // SpO2 based on Ratio
			ESpO2 = FSpO2 * ESpO2 + (1.0 - FSpO2) * SpO2; // SpO2 filtered

//         DEBUGPRINT(" SpO2 : %f", SpO2);
//         DEBUGPRINT(" R: %f", R);
			DEBUGPRINT("Ebpm : %d", Ebpm);
			DEBUGPRINT(" ESpO2 : %f\r\n", ESpO2);

			//reset sums and indices
			sumredrms = 0.0; 
			sumirrms = 0.0; 
			i = 0;
		}
	}			
}
#elif defined(I2C_MPU6050)
static const uint8_t expected_who_am_i = 0x68U; // !< Expected value to get from WHO_AM_I register.
static uint8_t       m_device_address;          // !< Device address in bits [7:1]
/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
void MPU6050_Init(void)
{
	uint8_t id;
	uint32_t SensorTimeout = SENSOR_TIMEOUT;
	
	stSensor.Flag = 0;	
	while(mpu6050_init(MPU6050_ADDRESS) == false){
		stSensor.Flag = 1;
		if((SensorTimeout--) == 0) break;
	}
	if(stSensor.Flag == 0){
		mpu6050_register_read(0x75U, &id, 1);
		DEBUGPRINT("mpu6050 id is 0x%02X \r\n",id);	
	}
	else{
		DEBUGPRINT("mpu6050 Init Error \r\n");	
	}
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
bool mpu6050_init(uint8_t device_address)
{   
  bool transfer_succeeded = true;
	
	uint8_t inData[7]={0x00,									//0x19
								0x00,												//0x1A
								0x03,												//0x6B
								0x10,												//0x1B
								0x00,												//0x6A
								0x32,												//0x37
								0x01};											//0x38
	uint8_t acc = 0x00;							
								
  m_device_address = (uint8_t)(device_address);

  // Do a reset on signal paths
  uint8_t reset_value = 0x04U | 0x02U | 0x01U; // Resets gyro, accelerometer and temperature sensor signal paths
  transfer_succeeded &= mpu6050_register_write(ADDRESS_SIGNAL_PATH_RESET, reset_value);
  
	
	//GYRO
	mpu6050_register_write(0x19 , inData[0]); 
	mpu6050_register_write(0x1A , inData[1]);
	mpu6050_register_write(0x6B , inData[2]);
	mpu6050_register_write(0x1B , inData[3]);
	mpu6050_register_write(0x6A , inData[4]);	

	mpu6050_register_write(0x37,  inData[5]);	
	mpu6050_register_write(0x38,  inData[6]); 
								
  mpu6050_register_write(0x1C,acc);
								
  // Read and verify product ID
  transfer_succeeded &= mpu6050_verify_product_id();

  return transfer_succeeded;
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
bool mpu6050_verify_product_id(void)
{
    uint8_t who_am_i;

    if (mpu6050_register_read(ADDRESS_WHO_AM_I, &who_am_i, 1)){
        if (who_am_i != expected_who_am_i){
            return false;
        }
        else{
            return true;
        }
    }
    else{
        return false;
    }
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
bool mpu6050_register_write(uint8_t register_address, uint8_t value)
{
    ret_code_t ret;
	  uint8_t w2_data[2];

    w2_data[0] = register_address;
    w2_data[1] = value;
	
	  ret = nrf_drv_twi_tx(&m_twi, m_device_address, w2_data, 2, true);
    if (NRF_SUCCESS != ret){
        return false;
    }
		return true;
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
bool mpu6050_register_read(uint8_t register_address, uint8_t * destination, uint8_t number_of_bytes)
{
    ret_code_t ret;
	
	  ret = nrf_drv_twi_tx(&m_twi, m_device_address, &register_address, 1, true);
    if (NRF_SUCCESS != ret){
        return false;
    }
	
	  ret = nrf_drv_twi_rx(&m_twi, m_device_address, destination, number_of_bytes);		
		if (NRF_SUCCESS != ret){
        return false;
    }
		return true;
}

/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
void MPU6050_ReadGyro(int16_t *pGYRO_X , int16_t *pGYRO_Y , int16_t *pGYRO_Z)
{
  uint8_t buf[6];    			
	
  mpu6050_register_read(MPU6050_GYRO_OUT,  buf, 6);
	
  *pGYRO_X = (buf[0] << 8) | buf[1];
	if(*pGYRO_X & 0x8000) *pGYRO_X-=65536;
		
	*pGYRO_Y= (buf[2] << 8) | buf[3];
  if(*pGYRO_Y & 0x8000) *pGYRO_Y-=65536;
	
  *pGYRO_Z = (buf[4] << 8) | buf[5];
	if(*pGYRO_Z & 0x8000) *pGYRO_Z-=65536;
}
			   
/*******************************************************************************
* Function Name : 
* Parameters    : None
* Return        : None
* Description   : 
*******************************************************************************/
void MPU6050_ReadAcc(int16_t *pACC_X , int16_t *pACC_Y , int16_t *pACC_Z)
{    
	uint8_t buf[6];    		
    
	mpu6050_register_read(MPU6050_ACC_OUT, buf, 6);
  *pACC_X = (buf[0] << 8) | buf[1];
	if(*pACC_X & 0x8000) *pACC_X-=65536;
		
	*pACC_Y= (buf[2] << 8) | buf[3];
  if(*pACC_Y & 0x8000) *pACC_Y-=65536;
	
  *pACC_Z = (buf[4] << 8) | buf[5];
	if(*pACC_Z & 0x8000) *pACC_Z-=65536;
}
#endif
